@@ -204,7 +204,7 @@ void initdesc(const char *descfile)
             {
               gpiterator *iter = &(da->iter);
               int niter=readnumber(dfile,buf,BUFFER_SIZE);
-              if (niter<3 && niter>4) die(err_desc,"unsupported iterator");
+              if (niter<3 || niter>4) die(err_desc,"unsupported iterator");
               readentry(dfile,buf,BUFFER_SIZE);
               iter->type=strtotype(buf);
               readentry(dfile,buf,BUFFER_SIZE);
