@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2005  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -22,9 +22,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 static int multi=0;
 static int newprivvar=1;
 extern int initnode;
-extern int optstrict;
-extern int linecount;
-extern int safecoeff;
 
 int newdecl(int flag, int t, int initval, int *v)
 {
@@ -219,9 +216,10 @@ void makeblocks3(int bl1, int bl2, int bl3, int n, int bseq, int aseq, int ret,
   restorectx(sav0);
 }
 
-static void affectval(int vn, int val, int *seq)
+void affectval(int vn, int val, int *seq)
 {
   ctxvar *v = ctxstack+vn;
+  if (val<0) die(-1,"Internal error: affectval");
   int valn=newnode(Fassign,newleaf(v->node),val);
   *seq=addseqright(*seq,valn);
 }
@@ -251,10 +249,20 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
         /*Make sure GEN objects are gerepilable.*/
         val = newnode(Ftag, newnode(Ftag, newsmall(0), Ggen), tv);
       if (decl==global)
-        fillvar(var,flag,tv,val);
+      {
+        int f=fillvar(var,flag,tv,-1);
+        if (autogc && ctype[tv]==Vgen)
+          affectval(f,val,seq);
+      }
       else
         pushvar(var,flag,tv,val);
       break;
+    case Findarg:
+      if (decl==function)
+        pushvar(tree[stack[i]].x,flag|(1<<Cindarg),mint,-1);
+      else
+        die(n,"invalid ~ in declaration");
+      break;
     case Fentry:
       switch(decl)
       {
@@ -270,10 +278,12 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
           val=newcall("_const_quote",newstringnode(entryname(stack[i]),-1));
           affectval(fillvar(stack[i],flag,mint,-1),val,seq);
         }
-        else if (autogc)
+        else
         {
+          int f = fillvar(stack[i],flag,mint,-1);
           /*Make sure (implicitly GEN) objects are gerepilable.*/
-          fillvar(stack[i],flag,mint, newsmall(0));
+          if (autogc)
+            affectval(f,  newsmall(0), seq);
         }
         break;
       case function:
@@ -302,15 +312,16 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
             int j, ret;
             int nb=listtostack(tree[x].x,Fmatrixelts,vars,STACKSZ," declaration",n);
             val = tree[stack[i]].y;
-            newdecl((1<<Cauto)|(1<<Cconst),Gvoid,GNIL,&ret);
+            newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&ret);
             *seq = addseqright(*seq, geninsertvar(val,ret));
             for (j=0;j<nb;j++)
-            {
-              checkisvar(vars[j],"my",&var,&tv);
-              if (tv==Gnotype)
-                tv=mint;
-              affectval(pushvar(var,flag,tv,-1),newcoeff(newleaf(ret),newsmall(j+1),-1),seq);
-            }
+              if (tree[vars[j]].f!=Fnoarg)
+              {
+                checkisvar(vars[j],"my",&var,&tv);
+                if (tv==Gnotype)
+                  tv=mint;
+                affectval(pushvar(var,flag,tv,-1),newcoeff(newleaf(ret),newsmall(j+1),-1),seq);
+              }
           } else
           {
             genequal(stack[i],"declaration",&var,&val,&tv);
@@ -342,6 +353,12 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
         break;
       }
       break;
+    case Fvararg:
+      if (decl==function && i==nb-1)
+        pushvar(tree[stack[i]].x,flag|(1<<Cvararg),Gvec,val);
+      else
+        die(n,"invalid variadic");
+      break;
     default:
       die(n,"Incorrect node %s in function declaration",
           funcname(tree[stack[i]].f));
@@ -411,7 +428,7 @@ void genblockdeffunc(int n)
   int seq=tree[n].y;
   const char *name=entryname(funcid);
   int args=tree[funcid].y;
-  int bl1,gltop=-1;
+  int bl1,gltop=-1,level=preclevel;
   int savcf,savnpv;
   int sav1,nodebl1;
   userfunc *ufunc;
@@ -469,6 +486,7 @@ void genblockdeffunc(int n)
   /*restore globalvar*/
   currfunc=savcf;
   newprivvar=savnpv;
+  preclevel=level;
 }
 
 int newvectoridx(int var, int ind)
@@ -551,27 +569,27 @@ void genblockaffmatrixl(int n, int x, int y, int z)
   aseq=newcall("for",stacktoargs(arg,4));
   makeblocks(bl1,bl2,n,bseq,aseq,vz,savx);
 }
-void genblockvector(int n, const char *typ)
+void genblockvector(int n, int t, const char *typ)
 {
   int arg[STACKSZ];
   int x=tree[n].x;
   int i,nb,ret;
   int aseq;
   nb=listtostack(x,Fmatrixelts,arg+1,STACKSZ-1,"Vector too long.",n);
-  newdecl(0,Gvec,-1,&ret);
+  newdecl(0,t,-1,&ret);
   arg[0]=newgetg(ret, newsmall(nb), typ);
   for(i=1;i<=nb;i++)
     arg[i]=geninsertvar(arg[i],newvectoridx(newleaf(ret),i));
   for(i=1;i<=nb;i++)
     genblock(arg[i],-1);
-  aseq=newcall("_makevec",stacktoargs(arg,nb+1));
+  aseq=newcall(t==Gvec?"_makevec":"_makevecsmall",stacktoargs(arg,nb+1));
   makeblock(newblock(),n,aseq,ret,s_ctx.n);
 }
 void genblockindex(int n)
 {
   int ret,z;
   int aseq;
-  if (n==GNOARG || tree[n].f==Fsmall)
+  if (n==GNORANGE || tree[n].f==Fsmall)
     return;
   newdecl((1<<Cconst),Gsmall,-1,&ret);
   z=newleaf(n);
@@ -623,18 +641,21 @@ genblockvecaff(int n)
 {
   int vars[STACKSZ];
   int x=tree[n].x, y=tree[n].y;
-  int aseq, ret, i;
+  int aseq, ret, i, savx;
   int nv=listtostack(tree[x].x,Fmatrixelts,vars,STACKSZ,"[,...,]=",n);
   genblock(y,n);
-  newdecl(1<<Cconst,Gvec,-1,&ret);
+  newdecl((1<<Cauto),Gempty,-1,&ret);
   aseq = geninsertvar(y,ret);
+  savx = s_ctx.n;
   for(i=0; i<nv; i++)
-  {
-    genblock(vars[i],n);
-    vars[i] = newnode(Fassign,vars[i],newcoeff(newleaf(ret),newsmall(i+1),-1));
-  }
+    if (tree[vars[i]].f!=Fnoarg)
+    {
+      genblock(vars[i],n);
+      ctxstack[getvarerr(getlvaluerr(vars[i]))].flag|=1<<Ccompo;
+      vars[i] = newnode(Fassign,vars[i],newcoeff(newleaf(ret),newsmall(i+1),-1));
+    }
   aseq = addseqright(aseq,stacktoseq(vars,nv));
-  makeblock(newblock(),n,aseq,ret,s_ctx.n);
+  makeblock(newblock(),n,aseq,ret,savx);
 }
 
 void genblockproto(int n, gpfunc *gp)
@@ -743,6 +764,7 @@ void genblock(int n, int p)
   int x,y;
   if (n<0)
     return;
+  filecount=tree[n].fileno;
   linecount=tree[n].lineno;
   x=tree[n].x;
   y=tree[n].y;
@@ -768,7 +790,7 @@ void genblock(int n, int p)
         tree[n]=tree[z];
       }
       else
-        genblockvector(n, "t_VEC");
+        genblockvector(n, Gvec, "t_VEC");
     break;
   case Fassign:
     x=detag(x);
@@ -918,40 +940,34 @@ void genblock(int n, int p)
       tree[n].f=Ffunction;
     }
   case Ffunction:/*fall through*/
-    if (x==OPn)
-    {
-      if (tree[y].f==Fsmall)
-      {
-        tree[n]=tree[y];
-        tree[n].x=-tree[n].x;
-        break;
-      }
-      else if (tree[y].f==Fconst)
-      {
-        value_t *val = value + tree[y].x;
-        if (val->type==CSTsmall || val->type==CSTsmallreal)
-        {
-          tree[n]=tree[y];
-          val->val.small=-val->val.small;
-          break;
-        }
-      }
-    }
-    else if (x==OPtrans && tree[y].f==Fvec)
+    if (x==OPtrans && tree[y].f==Fvec)
     {
       if (tree[y].x==-1)
       {
         int z=newgetgvec("t_COL");
         tree[n]=tree[z];
-        break;
       }
       else
       {
         tree[n]=tree[y];
-        genblockvector(n,"t_COL");
-        break;
+        genblockvector(n,Gvec,"t_COL");
       }
     }
+    else if(strcmp(value[x].val.str,"Vecsmall")==0 && tree[y].f==Fvec)
+    {
+      if (tree[y].x==-1)
+      {
+        int z=newgetgvec("t_VECSMALL");
+        tree[n]=tree[z];
+      }
+      else
+      {
+        tree[n]=tree[y];
+        genblockvector(n,Gvecsmall,"t_VECSMALL");
+      }
+
+    }
+    else
     {
       int nf=findfunction(value[x].val.str);
       gpfunc *gp = lfunc+currfunc;
@@ -970,7 +986,7 @@ void genblock(int n, int p)
         genblock(n,p);
       }
       else if (nf>=0 && lfunc[nf].iter)
-        genblockfunciter(n,p,lfunc+nf);
+        genblockfunciter(n,lfunc+nf);
       else if (nf>=0 && lfunc[nf].spec>0)
         genblockfuncspec(n,p,lfunc+nf);
       else if (nf>=0 && lfunc[nf].proto.code)
