@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2002  The PARI group.
+Copyright (C) 2002-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,13 +10,15 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "stack.h"
 
+int debug_stack;
+
 void ** stack_base(stack *s)
 {
   return (void **) ((unsigned long)s + (unsigned long)s->offset);
