@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2002-2003  The PARI group.
+Copyright (C) 2002-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -44,7 +44,10 @@ void mkfunc(int n, int p, int flag, int wr)
   const char *name=entryname(x);
   int nf=findfunction(name);
   if (nf==-1)
-    gp=lfunc+newuserfunc(name);
+  {
+    nf = newuserfunc(name);
+    gp = lfunc+nf;
+  }
   else
   {
     gp=lfunc+nf;
@@ -56,7 +59,8 @@ void mkfunc(int n, int p, int flag, int wr)
       int parent=ufunc->pdefnode;
       int node=ufunc->defnode;
       if (tree[node].y>=0)
-        warning(node,"Function %s redefined line %d",name,tree[n].lineno);
+        warning(node,"Function %s redefined in file %s, line %d",
+                name, srcfile[tree[n].fileno], tree[n].lineno);
       if (tree[parent].x==node)
         tree[parent].x=GNIL;
       else
@@ -71,13 +75,15 @@ void mkfunc(int n, int p, int flag, int wr)
 
 static void topfunc(int n, int p, int fun, int pfun, int nf, int wr)
 {
-  int y = tree[n].y;
+  int code = tree[n].y, arglist = tree[n].x;
+  int args = arglist>=0 ? tree[arglist].y : -1;
   int fl = wr<0?(1<<UFclosure):(1<<UFclosure)|(1<<UFstatic);
   if (fun==-1 || pfun==-1)
   {
     if(n!=nf) tree[n]=tree[nf];
     mkfunc(nf,p,0,wr);
-    gentopfunc(y,n,n,p);
+    gentopfunc(args,n,n,p);
+    gentopfunc(code,n,n,p);
     return;
   }
   if (tree[pfun].x==fun)
@@ -85,14 +91,16 @@ static void topfunc(int n, int p, int fun, int pfun, int nf, int wr)
     int seq = newnode(Fseq,nf,newleaf(fun));
     tree[fun] = tree[seq];
     mkfunc(nf,seq,fl,wr);
-    gentopfunc(y,n,nf,seq);
+    gentopfunc(args,n,nf,seq);
+    gentopfunc(code,n,nf,seq);
   }
   else
   {
     int seq = newnode(Fseq,tree[pfun].x,nf);
     tree[pfun].x = seq;
     mkfunc(nf,seq,fl,wr);
-    gentopfunc(y,n,nf,seq);
+    gentopfunc(args,n,nf,seq);
+    gentopfunc(code,n,nf,seq);
   }
 }
 
@@ -102,7 +110,7 @@ static void topfunclambda(int n, int p, int fun, int pfun, int wr)
   int nn = newanon();
   int nf = newnode(Fdeffunc,newnode(Ffunction,nn,x),y);
   int seq = newnode(Fentry,nn,-1);
-  topfunc(n,p,fun,pfun,nf,wr);
+  topfunc(nf,p,fun,pfun,nf,wr);
   if (fun>=0) tree[n] = tree[seq];
 }
 
@@ -112,12 +120,14 @@ static int topfuncproto(int n, int fun, int pfun, int nf)
   int nb=genlistargs(n,arg,0,STACKSZ);
   gpfunc *gp = lfunc+nf;
   gpwrap *gw = gp->wrap;
+  int nbwr = gw ? gw->nb: 0;
   const char *proto=gp->proto.code;
   int i=0;
-  int var=-1, seq, a, t, binf, wr;
+  int var=-1, seq, a, t, binf, wr, kvar=-1;
   char const *p=proto;
   char c;
   PPproto mod;
+  gpdesc *iter = lfunc[nf].iter;
   if (!proto) return 1;
   while((mod=parseproto(&p,&c)))
   {
@@ -127,9 +137,9 @@ static int topfuncproto(int n, int fun, int pfun, int nf)
       break;
     case PPstd:
       if (i>=nb || arg[i]==GNOARG)
-        die(n,"missing mandatorry argument");
+        die(n,"missing mandatory argument");
       a = arg[i++];
-      wr = i<=gw->nb ? gw->w[i-1]:-2;
+      wr = i<=nbwr ? gw->w[i-1]:-2;
       switch(c)
       {
         case 'G':
@@ -138,33 +148,36 @@ static int topfuncproto(int n, int fun, int pfun, int nf)
           break;
         case 'V':
           var = a;
-          killlistarg(n,a);
+          if (!iter)
+            killlistarg(n,a);
           break;
         case '=':
           genequal(a,lfunc[nf].gpname,&var,&binf,&t);
-          tree[a]=tree[binf];
+          kvar = a;
           break;
         case 'I':
         case 'E':
-          if (wr>=-1)
-          {
-            seq = newnode(Flambda,var,newleaf(a));
-            tree[a] = tree[seq];
-            topfunclambda(a, n, fun, pfun, wr);
-          }
+          if (iter || wr<-1)
+            break;
+        case 'J': /* Fall through */
+          if (kvar>=0 && !iter) { tree[kvar]=tree[binf]; kvar=-1; }
+          seq = newnode(Flambda,var,newleaf(a));
+          tree[a] = tree[seq];
+          topfunclambda(a, n, fun, pfun, wr);
           break;
       }
       break;
     case PPdefault:
       i++;
       a  = i<=nb ? arg[i-1]: GNOARG;
-      wr = i<=gw->nb ? gw->w[i-1]:-2;
+      wr = i<=nbwr ? gw->w[i-1]:-2;
       switch(c)
       {
         case 'I':
         case 'E':
-          if (a!=GNOARG && wr>=-1)
+          if (!iter && a!=GNOARG && (wr>=-1 || var==-1))
           {
+            if (kvar>=0) { tree[kvar]=tree[binf]; kvar=-1; }
             seq = newnode(Flambda,var,newleaf(a));
             tree[a]=tree[seq];
             topfunclambda(a, n, fun, pfun, wr);
@@ -229,8 +242,28 @@ void gentopfunc(int n, int p, int fun, int pfun)
     break;
   case Ffunction:
     {
-      int nf = findfunction(entryname(n));
-      if (nf>=0 && !lfunc[nf].iter && lfunc[nf].wrap && lfunc[nf].spec<0)
+      int nf;
+      if (x==OPn)
+      {
+        if (tree[y].f==Fsmall)
+        {
+          tree[n]=tree[y];
+          tree[n].x=-tree[n].x;
+          break;
+        }
+        else if (tree[y].f==Fconst)
+        {
+          value_t *val = value + tree[y].x;
+          if (val->type==CSTsmall || val->type==CSTsmallreal)
+          {
+            tree[n]=tree[y];
+            val->val.small=-val->val.small;
+            break;
+          }
+        }
+      }
+      nf = findfunction(entryname(n));
+      if (nf>=0 && lfunc[nf].spec<0)
         topfuncproto(n,fun,pfun,nf);
       gentopfunc(y,n,fun,pfun);
     }
